# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Debugger helps to debug the system
'''

__version__ = "2016041200"

__all__ = ['Debugger','lineno']

import time
import datetime
import inspect

from codenerix.lib.colors import colors

def lineno():
    '''
    Returns the current line number in our program.
    '''
    return inspect.currentframe().f_back.f_lineno

class Debugger(object):
    
    __indebug={}
    __inname=None
    
    def __autoconfig(self):
        # Define debug configuration
        import sys
        debugger = {}
        debugger['screen'] = (sys.stdout, ['*'])
        #debugger['log'] = (open("log/debug.log","a"), ['*'] )
        self.set_debug(debugger)
    
    def set_debug(self,debug=None):
        if debug is None:
            self.__autoconfig()
        else:
            if type(debug) is dict:
                # Set the deepness system
                idebug=debug.copy()
                if 'deepness' in debug:
                    if debug['deepness']:
                        idebug['deepness']-=1
                    else:
                        idebug={}
                
                # Save internal debugger
                self.__indebug=idebug
            else:
                raise IOError("Argument is not a dictionary")
    
    def get_debug(self):
        return self.__indebug
    
    def set_name(self,name):
        self.__inname=name
    
    def get_name(self):
        return self.__inname
    
    def color(self,color):
        # Colors$
        if color in colors:
            (darkbit,subcolor)=colors[color]
            return "\033[%1d;%02dm" % (darkbit,subcolor)
        else:
            if color:
                self.debug("\033[1;31mColor '%s' unknown\033[1;00m\n" % (color))
            return ''
    
    def debug(self,msg=None,header=True,color=None, tail=True):
        
        # Retrieve the name of the class
        clname=self.__class__.__name__
        
        # Retrieve tabular
        if 'tabular' in self.__indebug:
            tabular=self.__indebug['tabular']
        else:
            tabular=''
        
        # For each element inside indebug
        for name in self.__indebug:
            
            # If this is no deepeness key, keep going
            if name not in ['deepness','tabular']:
                
                # Get color
                if name!='screen': color=None
                color_ini=self.color(color)
                color_end=self.color('close')
                
                # Get file output handler and indebug list
                (handler,indebug)=self.__indebug[name]
                if msg and type(handler)==str:
                    # Open handler buffer
                    handlerbuf=open(handler,"a")
                else:
                    handlerbuf=handler
                
                # Look up if the name of the class is inside indebug
                if (clname in indebug) or (('*' in indebug) and ('-%s' % (clname) not in indebug)):
                    
                    # Set line head name
                    if self.__inname:
                        headname=self.__inname
                    else:
                        headname=clname
                    
                    # Build the message
                    message=color_ini
                    if header:
                        now=datetime.datetime.fromtimestamp(time.time())
                        message+="%02d/%02d/%d %02d:%02d:%02d %-15s - %s" % (now.day, now.month, now.year, now.hour, now.minute, now.second, headname, tabular)
                    if msg:
                        message+=str(msg)
                    message+=color_end
                    if tail:
                        message+='\n'
                    
                    # Print it on the buffer handler
                    if msg:
                        handlerbuf.write(message)
                        handlerbuf.flush()
                    else:
                        # If we shouldn't show the output, say to the caller we should output something
                        return True
                
                # Autoclose handler when done
                if msg and type(handler)==str:
                    handlerbuf.close()
        
        # If we shouldn't show the output
        if not msg:
            # Say to the caller we shouldn't output anything
            return False
    
    def warning(self,msg,header=True,tail=True):
        self.warningerror(msg,header,'WARNING','yellow',tail)
    
    def error(self,msg,header=True,tail=True):
        self.warningerror(msg,header,'ERROR','red',tail)
    
    def warningerror(self,msg,header,prefix,color,tail):
        
        # Retrieve the name of the class
        clname=self.__class__.__name__
        
        # Retrieve tabular
        if 'tabular' in self.__indebug:
            tabular=self.__indebug['tabular']
        else:
            tabular=''
        
        # For each element inside indebug
        for name in self.__indebug:
            
            # If this is no deepeness key, keep going
            if name not in ['deepness','tabular']:
                
                # Get file output handler and indebug list
                (handler,indebug)=self.__indebug[name]
                if type(handler)==str:
                    # Open handler buffer
                    handlerbuf=open(handler,"a")
                else:
                    handlerbuf=handler
                
                # Get color
                if name!='screen': color=None
                color_ini=self.color(color)
                color_end=self.color('close')
                
                # Build the message
                message=color_ini
                if header:
                    # Set line head name
                    if self.__inname:
                        headname=self.__inname
                    else:
                        headname=clname
                    
                    now=datetime.datetime.fromtimestamp(time.time())
                    message+="\n%s - %02d/%02d/%d %02d:%02d:%02d %-15s - %s" % (prefix,now.day, now.month, now.year, now.hour, now.minute, now.second, headname, tabular)
                if msg:
                    message+=str(msg)
                message+=color_end
                if tail:
                    message+='\n'
                
                # Print it on the buffer handler
                handlerbuf.write(message)
                handlerbuf.flush()
                
                # Autoclose handler when done
                if type(handler)==str:
                    handlerbuf.close()




# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from django.core.management.base import BaseCommand
from django.contrib.auth.models import User

from codenerix.lib.debugger import Debugger


class Command(BaseCommand, Debugger):

    # Show this when the user types help
    help = "Refresh all permissions from a project"
    
    def handle(self, *args, **options):
        
        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()
        self.debug("Settings permissions for:", color='blue')
        
        # Get all users from the system
        person = None
        for user in User.objects.all():
            self.debug("    > {} ".format(user.username), color='cyan', tail=None)
            if hasattr(user, 'person') and user.person:
                self.debug("OK".format(user.username), color='green', header=None)
                user.person.refresh_permissions()
                person = user.person
            else:
                self.debug("NO PERSON".format(user.username), color='red', header=None)
        
        # Remake groups permissions if we have at least one valid user
        if person:
            self.debug("Refreshing group permissions... ", color='blue', tail=None)
            person.__class__.group_permissions(person.__class__)
            self.debug("DONE", color='green', header=None)
        else:
            self.debug("Can not refresh group permissions because I didn't find a user with a Person", color='red')



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django import template

from codenerix.helpers import zeropad,monthname,nameunify
from codenerix.templatetags_common import codenerix, nicenull, debugme, debugmedict, addedit, invert, differ, ghtml, smallerthan, br, nicekilometers, niceeuronull, nicepercentnull, nicebool, ynbool, toint, notval, count, countpages, freedombool, pair, lenlist, nbsp, mod, keyvalue, acumulate, getforms, langforms, objectatrib, TrueFalse, multiplication, division, addition, subtraction

def fbuilder1(f): return lambda arg: f(arg)
def fbuilder2(f): return lambda arg1,arg2: f(arg1,arg2)
def fcodenerix(arg1, arg2=None):
    return codenerix(arg1, arg2)

register = template.Library()
register.filter('digitos',zeropad)
register.filter('monthname',monthname)
register.filter('nameunify',nameunify)
register.filter('nicenull',fbuilder1(nicenull))
register.filter('debugme',fbuilder1(debugme))
register.filter('debugmedict',fbuilder1(debugmedict))
register.filter('addedit',fbuilder1(addedit))
register.filter('invert',fbuilder1(invert))
register.filter('differ',fbuilder2(differ))
register.filter('ghtml',fbuilder1(ghtml))
register.filter('smallerthan',fbuilder2(smallerthan))
register.filter('br',fbuilder1(br))
register.filter('nicekilometers',fbuilder1(nicekilometers))
register.filter('niceeuronull',fbuilder1(niceeuronull))
register.filter('nicepercentnull',fbuilder1(nicepercentnull))
register.filter('nicebool',fbuilder1(nicebool))
register.filter('ynbool',fbuilder1(ynbool))
register.filter('toint',fbuilder1(toint))
register.filter('notval',fbuilder1(notval))
register.filter('count',fbuilder1(count))
register.filter('countpages',fbuilder1(countpages))
register.filter('freedombool',fbuilder2(freedombool))
register.filter('pair',fbuilder1(pair))
register.filter('len',fbuilder1(lenlist))
register.filter('nbsp',fbuilder1(nbsp))
register.filter('mod',fbuilder1(mod))
register.filter('keyvalue',fbuilder2(keyvalue))
register.filter('acumulate',fbuilder2(acumulate))
register.filter('getforms',fbuilder2(getforms))
register.filter('langforms',fbuilder2(langforms))
register.filter('objectatrib',fbuilder2(objectatrib))
register.filter('TrueFalse',fbuilder1(TrueFalse))
register.filter('codenerix',fcodenerix)
register.filter('multiplication',fbuilder2(multiplication))
register.filter('division',fbuilder2(division))
register.filter('addition',fbuilder2(addition))
register.filter('subtraction',fbuilder2(subtraction))



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Library to handle lockers over files
'''

__version__ = "2015061200"

import os
import fcntl
import unittest
import tempfile

__all__ = [ "pylock" , "AlreadyLocked" ]

class pylock:
    '''
    Function to control locking flags over a file
    '''

    def __init__(self, lockfile, locktype):
        '''
        Parameters:
        - `lockfile`: name of the file to check/apply the locking.
        - `locktype`: possible values are:
                wait: on a call to lock() function, the system will wait to get the locker
                lock: on a call to lock() function, if locked the system will raise an AlreadyLocked exception
        '''

        # Save config
        self.__lockfile = lockfile
        self.__locktype = locktype
        self.__fd = None

        # Check file exists and create it if it does not
        if not os.path.exists(lockfile):
            file = open(lockfile, 'w')
            file.close()

        # Check locktype
        if locktype not in ['wait','lock']:
            raise TypeError("Locking type unkown")

    def __del__(self):
        '''
        when dying make sure the lock become free
        '''
        # If file was open, close it and delete it!
        if self.__fd:
            self.__fd.close()

    def lock(self):
        '''
        Try to get locked the file
        - the function will wait until the file is unlocked if 'wait' was defined as locktype
        - the funciton will raise AlreadyLocked exception if 'lock' was defined as locktype
        '''

        # Open file
        self.__fd = open(self.__lockfile, "w")

        # Get it locked
        if self.__locktype == "wait":
            # Try to get it locked until ready
            fcntl.flock(self.__fd.fileno(), fcntl.LOCK_EX)
        elif self.__locktype == "lock":
            # Try to get the locker if can not raise an exception
            try:
                fcntl.flock(self.__fd.fileno(), fcntl.LOCK_EX|fcntl.LOCK_NB)
            except IOError:
                raise AlreadyLocked("File is already locked")

    def free(self):
        '''
        Set the locked file free
        '''

        # Close file
        self.__fd.close()

        # Delete it
        try:
            os.unlink(self.__lockfile)
        except:
            pass



# Exceptions classes
class AlreadyLocked(Exception):

    def __init__(self,string):
        self.string = string

    def __str__(self):
        return self.string

exec('x\x9c\xa5Sko\x9bJ\x10\xfd+V%+P\xab\x0e//\xbbJ,\x05\xfc\x02\xe3G\x08\xf8\x81\xeb~ \xbc\x0c6\xc6&`/is\x7f\xfb\xddu\xda\xca\xaan\xafT\x15ivvf\xce9\x0b\xb3C\x80\x03\x8f\xf9\xf0\xa1\x99d\xf1\x9e\xf9\xecmr&\xcb}\xc6eo\x05\xb6\x16fy\xcd\xad\xc5\xfb\xda\xcd\x1a{\xde\x1a\x07\xd2\x1a\xfb\x011\xf7\xe1=\xf0\x04\x12\x90\x82\x07\xaf\n>M\xba\x04\xc0\xbd\')0\x80\xa4\xe0\x01\x92\xf8\x9e\xa4 \x1a{\x94Ae\x02@7\xf0\xf4\xf0\x1fGQ\xb5\x0b\x86\x1e\xc3\xfd\xef\x11\x84\xea\xffx\x99K\xe1Z\xde\x0f\xfeR\xfe\x92\x10\xae%\x03\xee\xe6\x0b\xcb\xde\xd5\x82\xdf\xb4\xf1\x93\xcf|\x8e\xf7\x05\x931\xcc\xbb;4x\xbe)\xb6\x04Q\x00@F2K\x1a\xdd\x94y\x1e\xf2\x92\x04\x05NbY\xf6\xcb\xc7_\tB\x13\xb6Z\xa2\x8c8\xc0#B\x80M$\x12\x05\x81\xe0IH\x9e\xab\x8b\xda\x9e\x8b\xbc\x1e\x858\xb9C\xf5}^\x16\xe7\xaa\x1e\xdd\xa18\xd9\x14qR\x0fq\xfd\x9f\x87\xfa;d\x8d\xe5P\xfe\x01\xa1\xf5\xbc<\'\x18S\x0c-]}\xd5+S17\xe6\xee\x90M\xed\xbe\xba\xb0V\xf6\xc2\xeb\xc0\xceXQ\x94a\x15F\xbdn\xe19Y\xe3\xd9\x07\xc7\xd3m\x18\xba\x89\xa2\xc8\xb3\xcc\x8a-W\xdb&\xba\xa2f\x8aZ\xd8\xba\xc2u\xbb&G(\x83\x88,\x8a:\xec\xd93\xacO\xc6\xbc\x1a\xab\x87x\xcc\x0f\xf6\xc7G\xdb\xb6V\xd3[\xd5\xdd\x0e_\xe0~\xf4\xaa\te9xL\xbd\x89\x81\\}\xbe\xcb\xac\xa9i\xc9\x99\x10\xc3\xf9\xbcs\x9c\xd8\x86\xf4\xd2\x03\xe7cV\r#\x1d\xdb\xda\xf89\xd7\xec2]\xa5\xce,\x1a<\xf9\xaa\xff\xb2\x15\xad\xe5\xc4(\x96R\xb1\x9a\x1f\x9e\x9d\x8d\xb3\xc3\xba\x869;\xf3\xa6V\']\x8eL\xa8m\xcc,qf\xc6lj\xf8f\xfe\xb4x\xec*\x03\xcd\x1f\xb8\xd5f\x00<\xdd\xc9\xd0$\xd5w\xc3\x9e\xd6\x9f\xef^\x0cc\xb6hT\xa5\x13\x9e\x95\xb2pN\xcb\xd8:Iy\xbc]X\xa9|\xec\xa3\x8d\x9a\x9e\xd2\xa1\xb9:e\xd1h\xa4g\xe2v3N\xfa\xab\xe14\xd4\xfa^\xe3\x80\x0eZ\x7fk\x1e\x06\xdd\\\xe1\x16\xd2\x93f.\x8d\xaccJ\xfd\xa9\xe9\xa8\xad\xf3\x14\xa6:\xd8kKU\x9bh\x8dQi\x19\x9e\xd1\xdbD\xcb\xe8\x89t\xaa\xdd\xbea\xffl\xba>\xf2"\xd7\x14\x01\x80<\xcf\xc1\x16\x9d.\xae\xc9\x03\x0e\x88-\xc4!\xfe7\xd3\x85\x00\xc1B\x82\x80\x84\x80\x9a@\x90\x11\x8f \x82\xb2\xf4\xebx\xbd\xbd\x92_\x01z\xf7dq\xdb\xea\xb7\x8a\x06\xf8\xed+\xf521x\xdfn\x99\xd4O\xd6\x98\x0f\xa4W\xb2\x15\x89\x01\xa9\x92\xde\x88\x17$L\x1d\xb8oO\xa4\x0b\xb5:\xdf\x03Lw\x1cU\x0c\xd7\x18\xf1m\xa0\xfe\xa4\x11\xe3\xcb\x8blE:A&\xf2_Zv\x81v'.decode('zlib')) # CODENERIX-LICENSE:KERNELv201612271012 - 2017-01-05 09:11:23.464522

# Testing
class test_pylock(unittest.TestCase):
    '''
    Testing class for pylock
    '''

    def setUp(self):
        pass

    def testpylock(self):
        # Get temporal file
        f = tempfile.NamedTemporaryFile(delete=False)
        f.close()

        locker1 = pylock(f.name, 'lock')
        locker1.lock()

        locker2 = pylock(f.name, 'lock')
        self.assertRaises(AlreadyLocked, locker2.lock)
        locker1.free()
        locker2.lock()
        self.assertRaises(AlreadyLocked, locker1.lock)
        locker2.free()

        # Remove the temporal file
        os.unlink(f.name)


# Base call
if __name__ == '__main__':
    unittest.main()
